
#ifndef ROAMTYPES__HERE
#define ROAMTYPES__HERE 1

/*
 * roamtypes.h         typedefs/structs common to several roamstepN.c modules
 * by Mark Duchaineau (free but copyrighted, see LibGen/COPYING)
 *
 * 2002-08-07: teased out from roamstep3.c; added roam_*int64 for portability
 * 2002-08-08: corrected WIN32 to _WIN32 per Thatcher Ulrich's suggestion;
 *             modified diamonds to use parent/kid links rather than hash
 * 2002-08-11: added fields and structures for step 4 library
 *
 */


/* 64-bit signed and unsigned integers */

#ifdef _WIN32
typedef signed __int64 roam_int64;
typedef unsigned __int64 roam_uint64;
#else
typedef signed long long int roam_int64;
typedef unsigned long long int roam_uint64;
#endif

/* other int types */

typedef int roam_int32;
typedef unsigned int roam_uint32;
typedef short roam_int16;
typedef unsigned short roam_uint16;
typedef signed char roam_int8;
typedef unsigned char roam_uint8;


#define DMSTOREN 65536 /* number of diamonds in storage pool */


/* frustum cull bit masks */
#define CULL_ALLIN 0x3f
#define CULL_OUT   0x40


/*
 * per-diamond record (carefully packed for alignment)
 */

typedef struct roamdm_structdef roamdm_struct,*roamdm;

struct roamdm_structdef {
    float v[3];            /* vertex position                               */
    float r_bound;         /* radius of sphere bound squared (-1 if NEW)    */
    float r_error;         /* radius of pointwise error squared             */
    roamdm p[2];           /* diamond's parents (two corners)               */
    roamdm a[2];           /* other two corners (a[0]=quadtree parent)      */
    roamdm k[4];           /* diamond's kids                                */
    roamdm q0,q1;          /* prev and next links on queue or free list     */
    roam_int16 iq;         /* index in priority-queue array                 */
    roam_int8 i[2];        /* our kid index within each of our parents      */
    roam_int8 l;           /* level of resolution                           */
    roam_uint8 cull;       /* IN/OUT bits for frustum culling               */
    roam_uint8 flags;      /* misc. bit flags (splits etc)                  */
    roam_uint8 lockcnt;    /* number of references (0=FREE)                 */
    char padding[4];       /* filler for alignment                          */
};


#endif /* ifndef ROAMTYPES__HERE */

